import 'package:bip32_ed25519/api.dart';
import 'package:test/test.dart';
import 'package:durt/src/hd_wallet.dart';
import 'package:durt/src/dewif.dart';
import 'package:durt/src/cesium_wallet.dart';

void main() {
  test('Generate HD wallet with first pubkey', () {
    const String mnemonicLang = 'french';

    print('------------------\n HD Wallet Test\n------------------\n');
    var mnemonicTest = generateMnemonic(lang: mnemonicLang);
    expect(mnemonicTest.split(' ').length, 12);

    String mnemonic =
        "mauve victoire victoire cupide chaleur minimal punaise utopie plumage géologie quitter chaise";

    HdWallet _hdWallet = HdWallet.fromMnemonic(mnemonic, lang: mnemonicLang);
    Bip32KeyPair _key = Bip32KeyPair(
        signingKey: _hdWallet.rootSigningKey,
        verifyKey: _hdWallet.rootVerifyKey);

    Bip32KeyPair derivation0 = _hdWallet.derive(keys: _key, index: 0);
    String pubkey0 = _hdWallet.getPubkey(derivation0);

    String message = "blabla test";
    var signature = _hdWallet.sign(message, keypair: derivation0);
    bool isOK = _hdWallet.verifySign(message, signature, keypair: derivation0);

    print('Mnemonic: ' + mnemonic);
    print('Pubkey 0: ' + pubkey0);
    print('Is signature OK ? : ' + isOK.toString());

    expect(pubkey0, '5Njj4DLDfS5v5bZs61f11V3vgnHzUJpURvATFdY1RXJL');
    expect(isOK, true);
  });

  test('Create dewif and unlock it', () {
    print('\n------------------\n DEWIF Test\n------------------\n');

    Dewif _dewif = Dewif();
    var mnemonicRFC =
        "crop cash unable insane eight faith inflict route frame loud box vibrant";
    var _dewifDataRFC = _dewif.generateDewif(mnemonicRFC, 'toto titi tata',
        lang: 'english',
        dewifCurrencyCode: DEWIF_CURRENCY_CODE_G1_TEST,
        testRfc: true);
    print(_dewifDataRFC);

    String? decryptedDewifRFC;
    try {
      decryptedDewifRFC = _dewif.mnemonicFromDewif(
          _dewifDataRFC, 'toto titi tata',
          lang: 'english', dewifCurrencyCode: DEWIF_CURRENCY_CODE_G1_TEST);
      print('Unlock: ' + decryptedDewifRFC);
    } on ChecksumException {
      print('Bad secret code');
    } catch (e) {
      print(e);
    }
    expect(
        decryptedDewifRFC?.trim(),
        equals(
            'crop cash unable insane eight faith inflict route frame loud box vibrant'));
  });

  test('Cesium wallet creation and signature', () {
    var cesiumWallet = CesiumWallet('toto', 'tata');

    var message = 'tata cesium testy';

    var signature = cesiumWallet.sign(message);
    bool isOK = cesiumWallet.verifySign(message, signature);

    expect(cesiumWallet.pubkey, 'A6o88pwgb24Ri3N6UEGALLGhVAm7vrK8fJYHxmw91JSc');
    expect(isOK, true);
  });

  test('Cesium wallet DEWIF encryption and decryption', () {
    print('------------------\n Cesium DEWIF test\n------------------\n');
    var cesiumWallet = CesiumWallet('toto', 'tata');
    Dewif _dewif = Dewif();

    var _dewifDataTest = _dewif.generateCesiumDewif(cesiumWallet.seed, 'JKSPU',
        dewifCurrencyCode: DEWIF_CURRENCY_CODE_G1_TEST, test: true);
    print('Lock: ' + _dewifDataTest);

    expect(_dewifDataTest,
        'AAAAARAAAAEOAcVCma5x/ipOzcfViv8J5cdgASn7zedlNz6M/19wbE79aNvILr1CEyM664m8BjgLKeWiz1GYJSR14+ZY61U=');

    Uint8List? decryptedDewifTest;
    try {
      decryptedDewifTest = _dewif.cesiumSeedFromDewif(_dewifDataTest, 'JKSPU',
          dewifCurrencyCode: DEWIF_CURRENCY_CODE_G1_TEST);
      print('Unlock: ' + decryptedDewifTest.toString());
    } on ChecksumException {
      print('Bad secret code');
    } catch (e) {
      print(e);
    }
    expect(
        decryptedDewifTest,
        equals([
          218,
          9,
          63,
          33,
          79,
          188,
          150,
          119,
          73,
          112,
          60,
          104,
          121,
          153,
          74,
          219,
          187,
          169,
          221,
          222,
          100,
          88,
          195,
          12,
          84,
          214,
          238,
          116,
          63,
          37,
          238,
          124
        ]));

    var reCesiumWallet = CesiumWallet.fromSeed(decryptedDewifTest!);

    expect(
        reCesiumWallet.pubkey, 'A6o88pwgb24Ri3N6UEGALLGhVAm7vrK8fJYHxmw91JSc');
  });
}
